//Umrechnungen:

//Der Drehgeber liefert 500 Schritte pro Umdrehung (0..499).
//Entsprechend gilt:
//phi = position*2*PI/500 rad

//Eine Einheit der Zeitmarke T = 1/1125Hz (vergl. Zeitgeber)
//Entsprechend gilt:
//t = zeitmarke/1125 s

//Bei jedem Drehgeber-Interrupt, wird die Differenz zwischen
//aktueller und letzter Zeitmarke gebildet.
//und zwar dauer=neu-alt, bei positiver Drehrichtung und
//         dauer=alt-neu bei negativer.
//Da dazwischen nur ein Drehgeberschritt erfolgte, also 1/500 Umdrehung
//gilt fr die Winkelgeschwindigkeit omega:
//w =   (1*2*PI/500 rad)/(dauer/1125Hz) rad/s

class Drehgeber
{
    public:
        uint8_t akku;
        uint8_t register_save;
        int     position;
        bool    erstes_mal;
        bool    referenz_erreicht;
        int     kanal_c_position;
        int     icr1;
        
        void start()
        {
            DDRA = 0b00000000;
            DDRB = 0b00000000;
            DDRC = 0b00000000;
            DDRD = 0b00000000;

            //ber PC0 (bei muecB an PD6 verbunden), kann der Drehgeber
            //resetet werden:
            DDRC &= 0b11111110;
            PORTC |= 0b00000001; //Pullup (Low-Potential soll auslsend sein.)

            //PD7 ist Ausgang, um bei muecB Interrupt auszulsen (bei PD2 / INT0)
            DDRD |= 0b10000000;
            //PD7 hochziehen (fallende Flanke lst bei muecB Interrupt aus!)
            PORTD |= 0b10000000;

            akku              = 0;
            position          = 0;
            erstes_mal        = true;
            referenz_erreicht = false;
            kanal_c_position  = 0;

            DDRA = 0b11111111;
            DDRC |= 0b10000000;


            //PD6==ICP1==KanalA als Eingang 
            //und PD2==INT0==KanalB als Eingang: GENDERT ZU PD5 !!!!
            //PD3==INT1==KanalC
            DDRD &=  0b10010011;
            //interne Pullup-Widerstnde setzen:
            PORTD |= 0b01101100; 

//NICHT INT0
//            MCUCR |= (1<<ISC01);
//            GICR  |= (1<<INT0);
//            sei();

//...sondern ICP1:
			TCCR1B = (1<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (1<<CS12) | (0<<CS11) | (1<<CS10);
			TIMSK |= (1<<ICF1);
			sei(); //Interrupts frei
        }

        void reset()
        {
            akku              = 0;
            position          = 0;
            erstes_mal        = true;
            referenz_erreicht = false;
            kanal_c_position  = 0;
        }
};

Drehgeber drehgeber;

SIGNAL(SIG_INPUT_CAPTURE1)  //Capturing an ICP1 / PD6 von 
{
	drehgeber.icr1 = ICR1;  //auslesen, um Interrupt wieder frei zu setzen.

    /* Save global interrupt flag */
    drehgeber.register_save = SREG;
    /* Disable interrupts */
    cli();

    drehgeber.akku = PIND;
    if( (drehgeber.akku & 0b00100000) > 0 ) //KanalB PD5 untersuchen
        drehgeber.position--;
    else
        drehgeber.position++;

    drehgeber.position+=500;  //auf Bereich 0..499 beschrnken
    drehgeber.position%=500;

    if( drehgeber.erstes_mal && (drehgeber.akku & 0b00001000) > 0 ) //KanalC / PD3 untersuchen
    {
        drehgeber.kanal_c_position = drehgeber.position;
        drehgeber.erstes_mal = false;
        drehgeber.referenz_erreicht = true;
    }
    else if( !drehgeber.erstes_mal && (drehgeber.akku & 0b00001000) > 0 )
    {
        drehgeber.position = drehgeber.kanal_c_position;
        drehgeber.referenz_erreicht = true;
    }


    //Wert des Drehgebers ausgeben:
    //muecA    -> muecB
    //PA0..PA7 -> PC0..PC7
    //PC7      -> PD3 / INT1
    PORTA = drehgeber.position;
    PORTC = drehgeber.position>>1;  //Bit8 von pos kommt auf Bit7 von PORTC
    
    /* Restore global interrupt flag */
    SREG = drehgeber.register_save;
    /* Enable interrupts*/
    sei();

    //Einige Taktzyklen diesen Zustand halten:
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    //Signale an muecB, dass neuer Wert anliegt.
    //muecA -> muecB
    //PD7   -> INT0/PD2
    PORTD &= 0b01111111;

    //Einige Taktzyklen diesen Zustand halten:
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    
    //..dann wieder hochziehen:    
    PORTD |= 0b10000000;
}
