//Anstatt eine feste Zykluszeit im main-Programm
//festzusetzen, wird ein Zhler eingerichtet, der 
//bei einem berlauf des Timers2 inkrementiert.
//Timer2 wird mit einer Vorteilung von 32 prpariert
//Dann ist die Frequenz der Zhlschritte: 9216000/32  = 288000Hz
//Die Interruptfunktion SIGNAL(SIG_OVERFLOW2) wird dann mit 288000/256=1125Hz aufgerufen.
//D.h. Die Zeiterfassung funktioniert etwas genauer als mit einer Millisekunde.
//Der groe Zhler als unsigned long int hat einen Bereich von 0..4294967295
//Ohne berlauf des Timestamps lassen sich damit Zeitrume 
//von mehr als 1000Stunden abdecken.

#include <math.h>
#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

#include "Drehgeber.h"

SIGNAL(SIG_INTERRUPT0)
{
    cli();
    drehgeber.reset();

    //..und Werte ausgeben, da ja kein Capturing ausgelst wird:
    //Wert des Drehgebers ausgeben:
    //muecA    -> muecB
    //PA0..PA7 -> PC0..PC7
    //PC7      -> PD3 / INT1
    PORTA = drehgeber.position;
    PORTC = drehgeber.position>>1;  //Bit8 von pos kommt auf Bit7 von PORTC

    //Einige Taktzyklen diesen Zustand halten:
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    //Signale an muecB, dass neuer Wert anliegt.
    //muecA -> muecB
    //PD7   -> INT0/PD2
    PORTD &= 0b01111111;

    //Einige Taktzyklen diesen Zustand halten:
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    DDRB = 0b00000000;
    
    //..dann wieder hochziehen:    
    PORTD |= 0b10000000;
    sei();
}


int main(void)
{
    drehgeber.start();

    //Interrupt 0 aktivieren, um Drehgeber von auen resetten zu
    //knnen:
    DDRD &=  0b11111011;
    //interne Pullup-Widerstnde setzen:
    PORTD |= 0b00000100; 
    MCUCR |= (1<<ISC01);
    GICR  |= (1<<INT0);
    sei();


    while(true)
    {
        if( (PINC & 0b00000001) == 0 )
            drehgeber.reset();
    }

    return 0;
}
