//Anstatt eine feste Zykluszeit im main-Programm
//festzusetzen, wird ein Zhler eingerichtet, der 
//bei einem berlauf des Timers2 inkrementiert.
//Timer2 wird mit einer Vorteilung von 32 prpariert
//Dann ist die Frequenz der Zhlschritte: 9216000/32  = 288000Hz
//Die Interruptfunktion SIGNAL(SIG_OVERFLOW2) wird dann mit 288000/256=1125Hz aufgerufen.
//D.h. Die Zeiterfassung funktioniert etwas genauer als mit einer Millisekunde.
//Der groe Zhler als unsigned long int hat einen Bereich von 0..4294967295
//Ohne berlauf des Timestamps lassen sich damit Zeitrume 
//von mehr als 1000Stunden abdecken.

//**** REGELN lernen ***********************

#include <math.h>
#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

#include "Balancierer.h"
#include "Zeitgeber.h"
#include "Zyklus.h"
#include "Servo.h"
#include "Drehgeber.h"
#include "RS232.h"

//Interruptfunktion aus Drehgeberklasse:
//Regler einbauen!

volatile bool geregelt=false;
volatile int32_t regeln_P=0; //Nur vor P-Anteil
volatile int32_t regeln_I=0;
volatile int32_t regeln_D=0;
volatile int32_t regeln_V=0; //Verstrkung auf alles
volatile int32_t integral = 0;
volatile int32_t e_alt=0,e_neu=0,omega_akt=0;
    //Variablen in Modus 1 und 2 zur Drehung des Servos:
volatile int32_t pos=0;


SIGNAL(SIG_INTERRUPT0)
{
    cli();

    drehgeber.zeitmarke_alt = drehgeber.zeitmarke;
    drehgeber.zeitmarke = zeitgeber.zeitmarke;
    drehgeber.dauer = drehgeber.zeitmarke - drehgeber.zeitmarke_alt;

    drehgeber.position = PINC;
    drehgeber.position |=  ((PIND & 0b00001000)<<5); 
    drehgeber.position_alt = drehgeber.position;
        //drehgeber.position+=drehgeber.OFFSET;  //spter ev. dazu nehmen

    if(geregelt==true)
    {
            e_neu = 250 - drehgeber.position;
//            integral+=e_neu*drehgeber.dauer;
            integral/=2;
            integral+=e_neu*drehgeber.dauer;


//Stellwert inkrementell und damit als Geschwindigkeit definieren.
            pos-=  regeln_P*e_neu
                   +(regeln_I*integral)/10000L; //dt wichtig!;

            if(drehgeber.dauer!=0)
                pos-=
                 (regeln_D*(e_neu-e_alt)*100)
                 /drehgeber.dauer;
 pos/=10;
            e_alt = e_neu;

if(pos>0)
pos--;
if(pos<0)
pos++;



//Sttigung:
            if(pos>SERVO_HALB)
                pos=SERVO_HALB;
            if(pos<-SERVO_HALB)
                pos=-SERVO_HALB;

            OCR1A = pos+SERVO_MITTE;
            OCR1B = pos+SERVO_MITTE;
    }


    sei();
}

int main(void)
{
    int hilf;
    int test_pos1=0,test_pos2=1,test_pos3=2;
    uint32_t pause;
    //Variable fr aktuellen Betriebsmodus:
    int modus = 0;


    //Parameter, um den Aufschwingvorgang durchzufhren:
    //hin pause her
    //Es werden wegen der geplanten genetischen Alg.Opt. nur
    //unsigned char werte genommen und dann umgerechnet.

//sind hier fest:

    int auf_hin_amplitude = 224*8;
    unsigned char auf_hin_geschwindigkeit = 11;
    int auf_her_amplitude = 135;
    int auf_pause=249*20;
    int auf_pause_merk=auf_pause;



    uint32_t GUETE=0;
    int32_t zzz=0;
    uint32_t zeitmarke_alt=0;



//werden gesucht:




    DDRB &= 0b11111100;

    rs232.sendeText("**** Start des Balancierers ****\n\r");

    //Module starten:
    balancierer.start();
    zeitgeber.start();

    zyklus.setzeMaxZaehler(10);
    zyklus.start();

    drehgeber.start();
    servo.start();
    rs232.start();

    rs232.sendeText("**** Start des Balancierers ****\n\r");

    while(true)
    {
        switch(modus)
        {
            case 0:
                rs232.sendeText("MODUS 0 - Startmenue#\n\r");
                geregelt=false;
                PORTC=0;
                OCR1A = SERVO_MITTE;
                OCR1B = SERVO_MITTE;
                 geregelt=false;
                //Modus, bei dem auf die Eingabe eines neuen Befehls-Modus
                //gewartet wird:
              
                rs232.sendeText("Aufschwingversuch #\n\r");
                rs232.sendeText("CODE000 Geben Sie als Modus 1 an: #$\n\r");

                //Eingabe bei modus 0: 1
                modus = rs232.holeInteger(true);

                rs232.sendeText("\n\rSie haben Modus Nr.");
                rs232.sendeInteger(modus);
                rs232.sendeText(" gewaehlt. #\n\r");
            break;
            case 1:
                rs232.sendeText("MODUS 1 - Parametereingabe#\n\r");
                OCR1A = SERVO_MITTE;
                OCR1B = SERVO_MITTE;

                //Parametereingabe zum Aufschwingen
                rs232.sendeText("Parameter fuer Aufschwingphase angeben: #\n\r");

                rs232.sendeText("\n\rCODE010 P-Anteil/20 (0..255):#\n\r");
                regeln_P = rs232.holeInteger(true);
                regeln_P*=20;
rs232.sendeText("\n\rP-Anteil eingegeben:");
rs232.sendeInteger(regeln_P);
rs232.sendeText("\n\r");



//                regeln_P-=128;

                rs232.sendeText("\n\rCODE011 regeln_I/20 (0..255):#\n\r");
                regeln_I = rs232.holeInteger(true);
                regeln_I*=20;
//                regeln_I-=128;

                rs232.sendeText("\n\rCODE012 regeln_D/20 (0..255):#\n\r");
                regeln_D = rs232.holeInteger(true);
                regeln_D*=20;
//                regeln_D-=128;

//                rs232.sendeText("\n\rCODE013 regeln_V (0..255):#\n\r");
//                regeln_V = rs232.holeInteger(true);

                rs232.sendeText("O.K. # \n\r");
                auf_pause=0;
                modus++;
            break;
            case 2:
//                       auf_pause=0; vorher initialisieren
                   //rs232.sendeText("MODUS 9 - Pendel beruhigen#\n\r");
                   hilf = drehgeber.position;
                   if(hilf>=3 && hilf<=80)
                       pos=-100;
                   if(hilf<=497 && hilf>=420)
                       pos=100;
                   if(auf_pause>=5000)
                   {
                       if(pos>0)
                           hilf=-1;
                       else
                           hilf=1;
                       auf_pause=auf_pause_merk;
                       modus++;
                   }
                   auf_pause++;
                   OCR1A = pos+SERVO_MITTE;
                   OCR1B = pos+SERVO_MITTE;
            break;
            case 3:
                rs232.sendeText("MODUS 2 - Pendelruhe abwarten#\n\r");
                //Pendelruhe abwarten
                test_pos3 = test_pos2;
                test_pos2 = test_pos1;
                test_pos1 = drehgeber.position;
                for(pause=0;pause<50000000;pause++);
                if(test_pos1==test_pos2 && test_pos2==test_pos3)
                {
                    rs232.sendeText("O.K. Pendel in Ruhe. #\n\r");
                    modus++;
                }
            break;
            case 4:
                rs232.sendeText("MODUS 3 - Initialisieren fuer Aufschwingen#\n\r");
                //Initialisierungen fr Testfahrt:
                zeitgeber.reset();
                drehgeber.reset();
                pos=0;
                modus++;
            break;
            case 5:
            modus++;
                   //rs232.sendeText("MODUS 4 - Hinbewegen#\n\r");
                   pos+=auf_hin_geschwindigkeit;
                   if(pos>=auf_hin_amplitude)
                   {
                       auf_pause = auf_pause_merk;
                       modus++;
                   }
                   OCR1A = pos+SERVO_MITTE;
                   OCR1B = pos+SERVO_MITTE;
            break;
            case 6:
            if( (PINB & 0b00000011)>0 )
            {
                while( (PINB & 0b00000011)>0 );
                       auf_pause=0;
                       e_alt=0;
                       e_neu=0;
                       zzz=0;
                       zeitmarke_alt = zeitgeber.zeitmarke;
                       geregelt=true;                    
                       modus++;
            }
            rs232.sendeDatenIterativ();
            break;
            case 7:
            if( (PINB & 0b00000011)>0 )
                    modus++;
            break;
            case 8:
                   pos=0;
                   OCR1A = pos+SERVO_MITTE;
                   OCR1B = pos+SERVO_MITTE;
                   geregelt=false;                    
                   modus++;

                   GUETE = zeitgeber.zeitmarke - zeitmarke_alt;
                   rs232.sendeText("MODUS 9 - Guete ausgeben#\n\r");
                   rs232.sendeText("CODE100 GUETE=");
                   rs232.sendePositiveInteger(GUETE);
                   rs232.sendeText("#\n\r");
                   auf_pause=0;
                   modus++;
            break;
            case 9:
            modus=0;
//                       auf_pause=0; vorher initialisieren
                   //rs232.sendeText("MODUS 9 - Pendel beruhigen#\n\r");
                   hilf = drehgeber.position;
                   if(hilf>=3 && hilf<=80)
                       pos=-100;
                   if(hilf<=497 && hilf>=420)
                       pos=100;
                   if(auf_pause>=5000)
                   {
                       if(pos>0)
                           hilf=-1;
                       else
                           hilf=1;
                       auf_pause=auf_pause_merk;
                       modus++;
                   }
                   auf_pause++;
                   OCR1A = pos+SERVO_MITTE;
                   OCR1B = pos+SERVO_MITTE;
            break;
            case 10:
                   if(pos==0)
                       modus=0;
                   if(auf_pause>=200)
                   {
                       auf_pause=0;
                       pos+=hilf;
                   }
                   OCR1A = pos+SERVO_MITTE;
                   OCR1B = pos+SERVO_MITTE;
                   auf_pause++;
            break;
            case 11:
                rs232.sendeDatenIterativ();
            break;
        }
        if(geregelt==false)
            zyklus.warteBis_ms(1);
    }


    return 0;
}
