//Umrechnungen:

//Der Drehgeber liefert 500 Schritte pro Umdrehung (0..499).
//Entsprechend gilt:
//phi = position*2*PI/500 rad

//Eine Einheit der Zeitmarke T = 1/1125Hz (vergl. Zeitgeber)
//Entsprechend gilt:
//t = zeitmarke/1125 s

//Bei jedem Drehgeber-Interrupt, wird die Differenz zwischen
//aktueller und letzter Zeitmarke gebildet.
//und zwar dauer=neu-alt, bei positiver Drehrichtung und
//         dauer=alt-neu bei negativer.
//Da dazwischen nur ein Drehgeberschritt erfolgte, also 1/500 Umdrehung
//gilt fr die Winkelgeschwindigkeit omega:
//w =   (1*2*PI/500 rad)/(dauer/1125Hz) rad/s

class Drehgeber
{
    public:
        int     position;
        int     OFFSET;
        int     position_alt;
        uint32_t zeitmarke;
        uint32_t zeitmarke_alt;
        int     dauer;

        double getPHI()
        {
            //    phi = (2.0*M_PI*(double)drehgeber.position)/500.0;
            return 0.01256637061436*(double)position;
        }

        double getOMEGA()
        {
            if(dauer>1000)
                return 0.0;
            if(dauer<-1000)
                return 0.0;


            //Ein Schritt sind 2pi/500rad
            //Einheit von dauer ist 1/4500 s
            //dt = dauer*(1/4500) = dauer/4500
            //omega = (2.0*M_PI*4500.0)/(500.0*(double)drehgeber.dauer);
            return 56.54866776461628/(double)dauer;
        }

        void start()
        {
            OFFSET=0;
            position = 0;
            position_alt = 1;
            dauer             = EINE_SEKUNDE;

            //Dafr sorgen, dass die digitalen Eingnge,
            //ber die das Drehgebersignal von muecA kommt
            //auch als Eingnge konfiguriert sind und Pullups aktiv sind.
            DDRC  = 0b00000000;  //ganzer PORTC
            DDRD &= 0b11110111;  //PD3 von PORTD
            DDRD &= 0b11111011;  //PD2 von PORTD

            //PD6 hier bei muecB ist mit PC0 bei muecA verbunden.
            //Ein Low-Pegel soll den Drehgeberzhler reseten:
            DDRD |= 0b01000000;
            PORTD |= 0b01000000; //Pegel hochziehen, um kein Zurckstellen bei muecA auszulsen.

            MCUCR |= (1<<ISC01);
            GICR  |= (1<<INT0);
            sei();
        }

        void reset()
        {
            OFFSET=0;
            PORTD &= 0b10111111; 
            position = 0;
            position_alt = 1;
            dauer             = EINE_SEKUNDE;

            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;  

            PORTD |= 0b01000000;
        }

        void reset2()
        {
            PORTD &= 0b10111111; 
            OFFSET = 250 - position;
            position_alt = 1;
            dauer             = EINE_SEKUNDE;

            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;           
            DDRC  = 0b00000000;  

            PORTD |= 0b01000000;
        }

};

Drehgeber drehgeber;


