#define TAKTFREQUENZ 9216000
#define BAUDRATE 115200
#define PUFFERGROESSE 30

#define SENDE_ZEICHEN(ZEICHEN) while( !(UCSRA & (1<<UDRE)) );UDR = ZEICHEN

class RS232
{
	private:
		uint16_t baudregister;
        char  puffer[PUFFERGROESSE];
        int16_t  i,k;
        double   x,y;

        //Variablen, um Pendelzustand senden zu knnen:
        int position_alt;
 
        int hilf;
        uint32_t merker_drehg_zeitmarke;
        int      merker_drehg_position;
        int      merker_drehg_dauer;
        uint32_t merker_servo_zeitmarke;
        int      merker_servo_position;
        int      merker_servo_dauer;


        uint32_t zeitmarke_alt;

        int iteration;

	public:	    
        void start()
        {
            baudregister = TAKTFREQUENZ/8/BAUDRATE-1;
            UBRRH = (unsigned char) (baudregister>>8); //Setzen des HIGH-Bytes des Baudraten-Registers
            UBRRL = (unsigned char)  baudregister;     //Setzen des LOW -Bytes des Baudraten-Registers
            //Einschalten des Senders und des Empfngers
            UCSRB = (1<<TXEN) | (1<<RXEN);	        
            //Setzen des Nachrichtenformats: 8 Datenbits, 1 Stopbits
            UCSRC = (1<<URSEL)|(1<<UCSZ0)|(1<<UCSZ1);
            UCSRA = (1<<U2X);


            position_alt = 0;
 
            zeitmarke_alt=zeitgeber.zeitmarke;
            merker_drehg_zeitmarke=zeitgeber.zeitmarke;
            merker_drehg_position=0;
            merker_drehg_dauer=EINE_SEKUNDE;  //1 Sekunde (Dummy-Wert)
            merker_servo_zeitmarke=zeitgeber.zeitmarke;
            merker_servo_position=0;
            merker_servo_dauer=EINE_SEKUNDE;  //1 Sekunde (Dummy-Wert)
            iteration=0;
        }
    
        void sendeZeichen(uint8_t zeichen)
        {
            while( !(UCSRA & (1<<UDRE)) ); 
            UDR = zeichen;
        }

        uint8_t holeZeichen()
        {
			while(!(UCSRA & (1<<RXC)));
			return UDR;
        }

        void sendeText (char* text)
        {
            i=0;
            while(text[i]!='\0')
            {
                sendeZeichen(text[i]);								
                i++;
            }
        }
	
        char* holeText(bool reflect)
        {
            i=0;
            do
            {
                puffer[i]=holeZeichen();
                if(reflect==true)
                    sendeZeichen(puffer[i]);
                i++;
            } while(i<PUFFERGROESSE-1 && puffer[i-1]!='#');
            puffer[i-1]='\0';
            return puffer;
        }		

        void sendePositiveInteger(int32_t zahl)
        {
            i=0;            
            do
            {
                puffer[i]=48+zahl%10;
                zahl/=10;
                i++;
            } while(zahl>0);

            for(k=0;k<i;k++)
                sendeZeichen(puffer[i-k-1]);
        }

        void sendeInteger(int16_t zahl)
        {

            if (zahl<0)
            {
                sendeZeichen('-');
                zahl=-zahl;
            }
            sendePositiveInteger(zahl);
       }

       int16_t holeInteger(bool reflect)
       {				
           holeText(reflect);
           i=0;
           k=0;
           if(puffer[0]=='-')
               i++;
           while(i<PUFFERGROESSE && puffer[i]>=48 && puffer[i]<=57)
           {
               k*=10;
               k+=puffer[i]-48;
               i++;
           }
           if(puffer[0]=='-')
               k=-k;
           return k;
       }	

       void sendeDouble(double zahl)
       {
            if (zahl<0)
            {
                sendeZeichen('-');
                zahl=-zahl;
            }
            sendePositiveInteger((long)zahl);
            sendeZeichen('.');
            sendePositiveInteger((long)((zahl-(double)((int)zahl))*10000.0));
       }	

       double holeDouble(bool reflect)
       {
           holeText(reflect);
           i=0;
           k=0;
           if(puffer[0]=='-')
               i++;
           while(i<PUFFERGROESSE && puffer[i]>=48 && puffer[i]<=57)
           {
               k*=10;
               k+=puffer[i]-48;
               i++;
           }
           if(puffer[0]=='-')
               k=-k;
           x=(double)k;
           if(puffer[i]=='.')
           {
               i++;
               k=0;
               while(i<PUFFERGROESSE && puffer[i]>=48 && puffer[i]<=57)
               {
                   k*=10;
                   k+=puffer[i]-48;
                   i++;
               }                              
           }
           y=(double)k;
           while(y>1.0)
               y*=0.1;
           if(puffer[0]=='-')
               return x-y;
           return x+y;
       }

       //Gesendet werden folgende Spalten:
       //Spalte 0: 'N' oder 'R' fr normaler Wert oder Referenzwert (Kanal C) 

       //Spalte 1: Zeitmarke zd Drehgeber t = zd/EINE_SEKUNDE
       //Spalte 2: Position  pd Drehgeber phi = pd*2*PI/500
       //Spalte 3: Dauer     dd Drehgeber omega =  (1*2*PI/500)/(dd/EINE_SEKUNDE) 
       //(Dauer in Stampschritten seit letztem Interrupt mit Richtungsvorzeichen)
       
       //Spalte 4: Zeitmarke zs Servo t = zs/EINE_SEKUNDE
       //Spalte 5: Position  ps Servo phi = (ps-SERVO_MIN-SERVO_MIN/2)*0.25*PI/(SERVO_MIN/2)
       //Spalte 6: Dauer     ds Servo omega =  (1*0.25*PI/(SERVO_MIN/2))/(ds/EINE_SEKUNDE) 
       //(Dauer in Stampschritten seit letztem Interrupt mit Richtungsvorzeichen)


       void sendeDatenIterativ() //Jeder Aufruf vervollstndigt Sendung weiter
       {
           switch(iteration)
           {
               case 0:
                   //Falls Pendel unbewegt, nichts senden
                   hilf = drehgeber.position;
                   if( hilf == position_alt )
                   {
                       iteration=-1;
                   }
                   else
                   {
                       position_alt = merker_drehg_position;

                       //Momentaufnahme des Bewegungszustandes:
                       merker_drehg_zeitmarke  = drehgeber.zeitmarke;
                       merker_drehg_position   = drehgeber.position;
                       merker_drehg_dauer      = drehgeber.dauer;
                       merker_servo_zeitmarke  = servo.zeitmarke;
                       merker_servo_position   = servo.position;
                       merker_servo_dauer      = servo.dauer;
                   }
               break;
               case 1:
                   //Kenntlichmachung des Datenfragments als 
                   //zu speichernde Bewegungsdatenzeile:
                   sendeText("CODE999 ");
               break;
               case 2:
                   //Information senden, ob es sich um den Referenzpunkt
                   //handelt, der als einziger einen Low-Pegel
                   //bei Kanal C erzeugt, oder nicht:
                   SENDE_ZEICHEN('R');
                   SENDE_ZEICHEN(' ');
               break;
               case 3:
                   sendePositiveInteger(merker_drehg_zeitmarke);
               break;
               case 4:
                   SENDE_ZEICHEN(' ');
               break;
               case 5:
                   sendeInteger(merker_drehg_position);
               break;
               case 6:
                   SENDE_ZEICHEN(' ');
               break;
               case 7:
                   sendeInteger(merker_drehg_dauer);
               break;
               case 8:
                   SENDE_ZEICHEN(' ');
               break;
               case 9:
                   sendePositiveInteger(merker_servo_zeitmarke);
               break;
               case 10:
                   SENDE_ZEICHEN(' ');
               break;
               case 11:
                   sendeInteger(merker_servo_position);
               break;
               case 12:
                   SENDE_ZEICHEN(' ');
               break;
               case 13:
                   sendeInteger(merker_servo_dauer);
               break;
               case 14:
                   SENDE_ZEICHEN(' ');
                   SENDE_ZEICHEN('#');  //Abschluzeichen zum leichteren Parsen
                   SENDE_ZEICHEN('\n');
                   SENDE_ZEICHEN('\r');
               break;
               default:
                   iteration=-1;
               break;
           }
           iteration++;
       }
};

RS232 rs232;






