//Timer 2 als Zhler mit Vorteilung von 256 definieren:

//WGM20 = 0 WGM21 = 0 => Zhlbetrieb, zhlt stndig von 0 bis 255
//COM20 = 0 COM21 = 0 => Pin OC2 nicht verbunden
//COM20 = 1 COM21 = 1 => Pin OC2 lschen on Top, setzen bei Compare
//CS 22 21 20 : 000 gestoppt
//CS 22 21 20 : 001 Vorteilung durch    1
//CS 22 21 20 : 010 Vorteilung durch    8
//CS 22 21 20 : 011 Vorteilung durch   32
//CS 22 21 20 : 100 Vorteilung durch   64
//CS 22 21 20 : 101 Vorteilung durch  128
//CS 22 21 20 : 110 Vorteilung durch  256
//CS 22 21 20 : 111 Vorteilung durch 1024

//Anstatt eine feste Zykluszeit im main-Programm
//festzusetzen, wird ein Zhler eingerichtet, der 
//bei einem berlauf des Timers2 inkrementiert.
//Timer2 wird mit einer Vorteilung von 8 prpariert
//Dann ist die Frequenz der Zhlschritte: 9216000/8  = 1152000Hz
//Die Interruptfunktion SIGNAL(SIG_OVERFLOW2) wird dann mit 288000/256=4500Hz aufgerufen.
//D.h. Die Zeiterfassung funktioniert etwas genauer als mit ca. einer fnftel Millisekunde.
//Der groe Zhler als unsigned long int hat einen Bereich von 0..4294967295
//Ohne berlauf des Timestamps lassen sich damit Zeitrume 
//von mehr als 250Stunden abdecken.

//Definition der Zeitmarken-Zhlschritte fr eine Sekunde:
#define EINE_SEKUNDE 4500

class Zeitgeber
{
    private:
        bool zustand;
        
    public:
        //unsigned long int zeitmarke;
        uint32_t zeitmarke;

        void start()
        {            
            zustand = false;
            zeitmarke = 0;

            //Fr Kontrollpiepser:
            DDRD  |= 0b10000000;
            PORTD &= 0b01111111;

            TCCR2 = (1<<FOC2) | (0<<WGM20) | (0<<COM21) | (0<<COM20) | (0<<WGM21) | (0<<CS22) | (1<<CS21) | (0<<CS20);
            TCNT2=0; //Timer2 Zhlregister Null setzen.
            TIMSK |= (1<<TOIE2); //Interrupt durch Timer2 aktivieren
        }

        void reset()
        {
            zustand = false;
            zeitmarke = 0;
            TCNT2=0;
        }

        void warten()
        {
            //fclock=9216000Hz, Vorteilung 1024, TCNT2<90 =>  100Hz
            //fclock=9216000Hz, Vorteilung  256, TCNT2<36 => 1000Hz
            //fclock=9216000Hz, Vorteilung    8, TCNT2<128 => 9000Hz
            while(TCNT2<128); 
            TCNT2=0;
        }

        void piepsen()  //Gibt auf Lautsprecher Rechtecksignal halber Timer-Frequenz aus.
        {
            if(zustand==true)
                PORTD |= 0b10000000;
            else
                PORTD &= 0b01111111;
            zustand = !zustand;
        }
};

Zeitgeber zeitgeber;

SIGNAL(SIG_OVERFLOW2)
{
    zeitgeber.zeitmarke++;
    //zeitgeber.piepsen(); //Zur Kontrolle PD7 mit Lautsprecher toggeln => 1125Hz/2 hrbar
}
